from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import Callable

from langchain.schema import BaseMessage

from llm.direction import TokenDirection
from llm.types import LLMClientClass, LLMClientInstance


@dataclass
class ModelConfig:
    """Конфигурация для конкретной модели"""

    client_class: LLMClientClass
    token_counter: Callable[[list[BaseMessage], str, LLMClientInstance], int]
    pricing: dict[TokenDirection, float]
    moderation: Callable[[list[BaseMessage], LLMClientInstance], None] | None = None
    test_connection: Callable[[LLMClientInstance], bool | None] | None = None


class BaseProvider(ABC):
    """Базовый класс для провайдеров LLM"""

    def __init__(self, usd_rate: float) -> None:
        self.usd_rate = usd_rate

    @property
    @abstractmethod
    def name(self) -> str:
        """Название провайдера

        Returns:
            str: Название провайдера
        """

    @abstractmethod
    def get_models(self) -> dict[str, ModelConfig]:
        """Возвращает словарь моделей провайдера

        Returns:
            dict[str, ModelConfig]: Словарь моделей провайдера
        """

    def has_model(self, model_name: str) -> bool:
        """Проверяет, поддерживается ли модель провайдером

        Args:
            model_name (str): Название модели

        Returns:
            bool: True, если модель поддерживается, False в противном случае
        """
        return model_name in self.get_models()

    def get_model_config(self, model_name: str) -> ModelConfig:
        """Получает конфигурацию модели

        Args:
            model_name (str): Название модели

        Returns:
            ModelConfig: Конфигурация модели
        """
        models = self.get_models()
        if model_name not in models:
            raise ValueError(f'Model {model_name} not found in {self.name}')
        return models[model_name]



import asyncio

from example.common_imports import *  # noqa: F403
from llm.service import LLMService


async def test() -> None:
    llm = await LLMService.create(grok_3_mini.to_dict())  # noqa: F405
    stream = await llm.astream(message='Кратко расскажи что такое Python')
    async for chunk in stream:
        print(chunk, end='', flush=True)

    print('\n\n')
    print(len(stream.full_text))
    print(llm.usage)
    print(llm.chat_json)


async def main() -> None:
    await test()


if __name__ == '__main__':
    asyncio.run(main())



from packaging import version

try:
    import pydantic

    PYDANTIC_VERSION = version.parse(pydantic.__version__)
except ImportError:
    PYDANTIC_VERSION = version.parse('0.0.0')

IS_PYDANTIC_V1 = PYDANTIC_VERSION.major == 1
IS_PYDANTIC_V2 = PYDANTIC_VERSION.major == 2


def is_pydantic_instance(obj) -> bool:
    """
    Проверяет, является ли переданный объект экземпляром Pydantic модели.

    Поддерживает:
        - pydantic.BaseModel в Pydantic 1.x
        - pydantic.BaseModel в Pydantic 2.x
        - pydantic.v1.BaseModel в Pydantic 2.x

    Args:
        obj: Объект для проверки

    Returns:
        bool: True, если объект является экземпляром Pydantic модели, иначе False
    """
    # Если объект None или не является объектом класса, возвращаем False
    if obj is None:
        return False

    if IS_PYDANTIC_V1:
        from pydantic import BaseModel as BaseModelV1

        return isinstance(obj, BaseModelV1)

    elif IS_PYDANTIC_V2:
        from pydantic import BaseModel as BaseModelV2
        from pydantic.v1 import BaseModel as BaseModelV1

        return isinstance(obj, (BaseModelV1, BaseModelV2))

    return False



from langchain_google_genai import ChatGoogleGenerativeAI

from llm.direction import TokenDirection
from llm.providers._base import BaseProvider, ModelConfig
from llm.test_connections import TestConnections
from llm.token_counters import TokenCounterFactory


class GoogleProvider(BaseProvider):
    """Провайдер для моделей Google"""

    @property
    def name(self) -> str:
        return 'Google'

    def get_models(self) -> dict[str, ModelConfig]:
        """Возвращает конфигурации всех моделей Google"""
        return {
            'gemini-2.0-flash-001': ModelConfig(
                client_class=ChatGoogleGenerativeAI,
                token_counter=TokenCounterFactory().create_google_counter(),
                test_connection=TestConnections().google,
                pricing={
                    TokenDirection.ENCODE: 0.10 / 1_000_000,
                    TokenDirection.DECODE: 0.40 / 1_000_000,
                },
            ),
            'gemini-2.5-flash': ModelConfig(
                client_class=ChatGoogleGenerativeAI,
                token_counter=TokenCounterFactory().create_google_counter(),
                test_connection=TestConnections().google,
                pricing={
                    TokenDirection.ENCODE: 0.30 / 1_000_000,
                    TokenDirection.DECODE: 1.00 / 1_000_000,
                },
            ),
            'gemini-2.5-pro-preview-06-05': ModelConfig(
                client_class=ChatGoogleGenerativeAI,
                token_counter=TokenCounterFactory().create_google_counter(),
                test_connection=TestConnections().google,
                pricing={
                    TokenDirection.ENCODE: 2.50 / 1_000_000,
                    TokenDirection.DECODE: 15.00 / 1_000_000,
                },
            ),
        }



import time
from http import HTTPStatus
from typing import Optional, Self
from xml.etree import ElementTree

import aiohttp


class CBRRate:
    _instance: Optional['CBRRate'] = None
    USD_CURRENCY_ID = 'R01235'
    CBR_DAILY_RATE_URL = 'https://www.cbr.ru/scripts/XML_daily.asp'
    RATE_REFRESH_BUFFER = 60 * 10

    def __new__(cls) -> Self:
        if cls._instance is None:
            cls._instance = super(CBRRate, cls).__new__(cls)
            cls._instance.cached_rate = None
            cls._instance.cached_time = None
        return cls._instance

    async def get_usd_rate(self) -> float:
        current_time = int(time.time())
        if self._rate_needs_refresh(current_time):
            new_rate = await self._fetch_usd_rate()
            if new_rate is not None:
                self.cached_time = current_time
                self.cached_rate = new_rate
        return self.cached_rate

    def _rate_needs_refresh(self, current_time: int) -> bool:
        return (
            self.cached_rate is None
            or self.cached_time is None
            or current_time >= self.cached_time + self.RATE_REFRESH_BUFFER
        )

    async def process_response(self, response: aiohttp.ClientResponse) -> float | None:
        if response.status == HTTPStatus.OK:
            content = await response.text()
            root = ElementTree.fromstring(content)
            for valute in root.findall('Valute'):
                if valute.get('ID') == self.USD_CURRENCY_ID:
                    rate = valute.find('Value').text
                    return float(rate.replace(',', '.'))
        return None

    async def _fetch_usd_rate(self) -> float | None:
        timeout = aiohttp.ClientTimeout(
            total=60,
            connect=10,
            sock_read=30,
        )
        connector = aiohttp.TCPConnector(
            limit=100,
            limit_per_host=30,
            keepalive_timeout=30,
            enable_cleanup_closed=True,
            force_close=False,
        )
        async with aiohttp.ClientSession(
            timeout=timeout,
            connector=connector,
        ) as session:
            async with session.get(
                url=self.CBR_DAILY_RATE_URL,
                ssl=False,
            ) as response:
                return await self.process_response(response)



from langchain_gigachat import GigaChat

from llm.direction import TokenDirection
from llm.providers._base import BaseProvider, ModelConfig
from llm.test_connections import TestConnections
from llm.token_counters import TokenCounterFactory


class GigaChatProvider(BaseProvider):
    """Провайдер для моделей GigaChat"""

    @property
    def name(self) -> str:
        return 'GigaChat'

    def get_models(self) -> dict[str, ModelConfig]:
        """Возвращает конфигурации всех моделей GigaChat"""
        return {
            'GigaChat': ModelConfig(
                client_class=GigaChat,
                token_counter=TokenCounterFactory().create_gigachat_counter(),
                test_connection=TestConnections().gigachat,
                pricing={
                    # 5_000 рублей / 25_000_000 токенов / курс доллара
                    TokenDirection.ENCODE: 5_000 / 25_000_000 / self.usd_rate,
                    TokenDirection.DECODE: 5_000 / 25_000_000 / self.usd_rate,
                },
            ),
            'GigaChat-2': ModelConfig(
                client_class=GigaChat,
                token_counter=TokenCounterFactory().create_gigachat_counter(),
                test_connection=TestConnections().gigachat,
                pricing={
                    # 5_000 рублей / 25_000_000 токенов / курс доллара
                    TokenDirection.ENCODE: 5_000 / 25_000_000 / self.usd_rate,
                    TokenDirection.DECODE: 5_000 / 25_000_000 / self.usd_rate,
                },
            ),
            'GigaChat-Pro': ModelConfig(
                client_class=GigaChat,
                token_counter=TokenCounterFactory().create_gigachat_counter(),
                test_connection=TestConnections().gigachat,
                pricing={
                    # 10_500 рублей / 7_000_000 токенов / курс доллара
                    TokenDirection.ENCODE: 10_500 / 7_000_000 / self.usd_rate,
                    TokenDirection.DECODE: 10_500 / 7_000_000 / self.usd_rate,
                },
            ),
            'GigaChat-2-Pro': ModelConfig(
                client_class=GigaChat,
                token_counter=TokenCounterFactory().create_gigachat_counter(),
                test_connection=TestConnections().gigachat,
                pricing={
                    # 10_500 рублей / 7_000_000 токенов / курс доллара
                    TokenDirection.ENCODE: 10_500 / 7_000_000 / self.usd_rate,
                    TokenDirection.DECODE: 10_500 / 7_000_000 / self.usd_rate,
                },
            ),
            'GigaChat-Max': ModelConfig(
                client_class=GigaChat,
                token_counter=TokenCounterFactory().create_gigachat_counter(),
                test_connection=TestConnections().gigachat,
                pricing={
                    # 15_600 рублей / 8_000_000 токенов / курс доллара
                    TokenDirection.ENCODE: 15_600 / 8_000_000 / self.usd_rate,
                    TokenDirection.DECODE: 15_600 / 8_000_000 / self.usd_rate,
                },
            ),
            'GigaChat-2-Max': ModelConfig(
                client_class=GigaChat,
                token_counter=TokenCounterFactory().create_gigachat_counter(),
                test_connection=TestConnections().gigachat,
                pricing={
                    # 15_600 рублей / 8_000_000 токенов / курс доллара
                    TokenDirection.ENCODE: 15_600 / 8_000_000 / self.usd_rate,
                    TokenDirection.DECODE: 15_600 / 8_000_000 / self.usd_rate,
                },
            ),
        }



import asyncio

from pydantic import BaseModel, Field

from example.common_imports import *  # noqa: F403
from llm.service import LLMService


class RelatedConceptOutput(BaseModel):
    """Новый термин и его сила связи с исходным термином."""

    title: str = Field(..., description='Название термина')
    length: int = Field(..., description='Сила связи')


class RelatedConceptListOutput(BaseModel):
    """Новые термины и их сила связи с исходным термином."""

    concepts: list[RelatedConceptOutput]


SYSTEM_PROMPT = (
    'Тебе дано понятие школьной программы: "Молекула". Сгенерируй ровно "5" понятий '
    'школьной программы, наиболее близких к этому понятию.'
)


async def test() -> None:
    llm = await LLMService.create(grok_3_mini.to_dict())  # noqa: F405
    structured_llm = await llm.with_structured_output(RelatedConceptListOutput)
    result = await structured_llm.ainvoke(message=SYSTEM_PROMPT)
    print(result)
    print(structured_llm.usage)
    print(structured_llm.usd_rate)
    print(structured_llm.chat_json)


async def main() -> None:
    await test()


if __name__ == '__main__':
    asyncio.run(main())



from http import HTTPStatus
from urllib.parse import urljoin

import aiohttp

from llm.types import LLMClientInstance


# TODO: Выполнять запросы самостоятельно, или использовать нативные библиотеки?
class TestConnections:
    @staticmethod
    async def _send_request(url: str, headers: dict[str, str]) -> bool:
        timeout = aiohttp.ClientTimeout(
            total=60,
            connect=10,
            sock_read=30,
        )
        connector = aiohttp.TCPConnector(
            limit=100,
            limit_per_host=30,
            keepalive_timeout=30,
            enable_cleanup_closed=True,
            force_close=False,
        )
        async with aiohttp.ClientSession(
            timeout=timeout,
            connector=connector,
        ) as session:
            async with session.get(
                url=url,
                headers=headers,
                ssl=False,
            ) as response:
                if response.status == HTTPStatus.OK:
                    return True
                return False

    @staticmethod
    async def openai(client: LLMClientInstance | None = None) -> bool:
        api_key = client.openai_api_key._secret_value

        base_url = str(client.root_async_client.base_url)
        path = 'models'
        full_url = urljoin(base_url, path)

        headers = {'Authorization': f'Bearer {api_key}'}

        return await TestConnections()._send_request(full_url, headers)

    @staticmethod
    async def anthropic(client: LLMClientInstance | None = None) -> bool:
        api_key = client.anthropic_api_key._secret_value

        base_url = client.anthropic_api_url
        path = 'v1/models'
        full_url = urljoin(base_url, path)

        headers = {
            'x-api-key': api_key,
            'anthropic-version': '2023-06-01',
        }

        return await TestConnections()._send_request(full_url, headers)

    @staticmethod
    async def gigachat(client: LLMClientInstance | None = None) -> bool:
        try:
            return bool(await client.aget_models())
        except Exception:
            return False

    @staticmethod
    async def google(client: LLMClientInstance | None = None) -> bool:
        api_key = client.google_api_key._secret_value

        url = 'https://generativelanguage.googleapis.com/v1beta/models'

        headers = {
            'x-goog-api-key': api_key,
            'Content-Type': 'application/json',
        }

        return await TestConnections()._send_request(url, headers)

    @staticmethod
    async def xai(client: LLMClientInstance | None = None) -> bool:
        api_key = client.xai_api_key._secret_value

        url = 'https://api.x.ai/v1/models'

        headers = {
            'Authorization': f'Bearer {api_key}',
            'Content-Type': 'application/json',
        }

        return await TestConnections()._send_request(url, headers)

    @staticmethod
    async def deepseek(client: LLMClientInstance | None = None) -> bool:
        api_key = client.api_key._secret_value

        url = client.api_base
        path = 'models'
        full_url = urljoin(url, path)

        headers = {'Authorization': f'Bearer {api_key}'}

        return await TestConnections()._send_request(full_url, headers)

    @staticmethod
    async def cerebras(client: LLMClientInstance | None = None) -> bool:
        api_key = client.cerebras_api_key._secret_value

        url = client.cerebras_api_base
        path = 'models'
        full_url = urljoin(url, path)

        headers = {'Authorization': f'Bearer {api_key}'}

        return await TestConnections()._send_request(full_url, headers)

    @staticmethod
    async def openrouter(client: LLMClientInstance | None = None) -> bool:
        return await TestConnections().openai(client)



import inspect
import json
from typing import Any, AsyncGenerator, Self

from langchain.schema import AIMessage, BaseMessage
from pydantic import BaseModel

from llm.billing import BillingDecorator, StreamBillingDecorator
from llm.cbr.cbr import CBRRate
from llm.counter import TokenCounter
from llm.model_registry import ModelRegistry
from llm.prepare_chat import PrepareChat
from llm.types import LLMClientInstance
from llm.usage import TokenUsage


class StreamResult:
    def __init__(
        self,
        generator: AsyncGenerator[str, None],
        get_full_text: callable = None,
    ):
        self._generator = generator
        self._get_full_text = get_full_text

    def __aiter__(self):
        return self

    async def __anext__(self):
        try:
            chunk = await self._generator.__anext__()
            return chunk
        except StopAsyncIteration:
            raise

    @property
    def full_text(self) -> str:
        """Возвращает полный текст. Доступен только после завершения итерации."""
        if self._get_full_text:
            return self._get_full_text()
        return ''


# TODO: Исправить подсчет токенов для with_structured_output. Сейчас токены считаются
# просто из выходного текста, а нужно считать по тому, что было вызвано в модели
# (function или tool или что-то еще)
# TODO: Добавить возможность использования всех моделей провайдера, или кастомных
class LLMService:
    def __init__(self, config: dict, usd_rate: float = None) -> None:
        self.config = config
        self.usd_rate = usd_rate

        self.model_registry = ModelRegistry(usd_rate, config)
        self.client: LLMClientInstance = self.model_registry.init_client()

        self.usage = TokenUsage()
        self.counter = TokenCounter(
            model_name=config.get('model'),
            usage=self.usage,
            model_registry=self.model_registry,
        )

        self.__is_structured_output = False

        self.__chat_history: list[BaseMessage] = []
        self.__last_response: AIMessage | None = None

    @classmethod
    async def create(cls, config: dict) -> Self:
        # Получаем курс доллара
        cbr = CBRRate()
        usd_rate = await cbr.get_usd_rate()

        # Создаем экземпляр класса с уже полученным курсом
        instance = cls(config, usd_rate)
        return instance

    async def __moderation_check(
        self,
        moderation: bool,
        chat_for_model: list[BaseMessage],
    ) -> None:
        if moderation:
            await self.model_registry.get_moderation(
                self.config.get('model'),
                chat_for_model,
            )

    def __serialize_structured_result(self, result: BaseModel | dict[str, Any]) -> str:
        if hasattr(result, 'model_dump'):
            return result.model_dump_json()
        elif isinstance(result, dict):
            return json.dumps(result, ensure_ascii=False)
        else:
            return str(result)

    def _save_chat_history(
        self, chat_for_model: list[BaseMessage], answer: AIMessage
    ) -> None:
        """Сохраняет историю чата и ответ модели."""
        self.__chat_history = chat_for_model.copy()
        self.__last_response = answer

    @property
    def chat_json(self) -> str | None:
        """Возвращает историю чата в JSON формате."""
        if not self.__chat_history or not self.__last_response:
            return None

        result = []
        for message in self.__chat_history:
            result.append(
                {
                    'type': message.type,
                    'content': message.content,
                },
            )
        result.append(
            {
                'type': self.__last_response.type,
                'content': self.__last_response.content,
            },
        )
        return json.dumps(result, ensure_ascii=False)

    async def test_connection(self) -> bool | None:
        return await self.model_registry.get_test_connections(self.config.get('model'))

    async def ainvoke(
        self,
        chat_history: list[dict[str, str] | BaseMessage] | None = None,
        system_prompt: str | BaseMessage | None = None,
        message: str | BaseMessage | None = None,
        moderation: bool = False,
        **kwargs,
    ) -> str:
        chat_for_model = PrepareChat(chat_history, system_prompt, message)

        await self.__moderation_check(moderation, chat_for_model)

        billing_invoke = BillingDecorator(self.client.ainvoke, self.counter)
        result = await billing_invoke(input=chat_for_model, **kwargs)

        if self.__is_structured_output:
            content = self.__serialize_structured_result(result)
            ai_message = AIMessage(content=content)
            self._save_chat_history(chat_for_model, ai_message)
            return result

        self._save_chat_history(chat_for_model, result)
        return result.content

    async def with_structured_output(
        self,
        schema: dict | type,
        *,
        method: str = None,
        include_raw: bool = None,
        strict: bool = None,
        tools: list = None,
        **kwargs: Any,
    ) -> Self:
        # Создаем новый экземпляр
        new_instance = self.__class__(self.config, self.usd_rate)

        # Получаем сигнатуру метода провайдера
        provider_method = self.client.with_structured_output
        sig = inspect.signature(provider_method)
        supported_params = set(sig.parameters.keys())

        # Собираем только явно переданные аргументы, которые поддерживает провайдер
        provider_kwargs = {'schema': schema}

        # Добавляем только явно заданные опциональные параметры
        optional_params = {
            'method': method,
            'include_raw': include_raw,
            'strict': strict,
            'tools': tools,
        }

        for param_name, param_value in optional_params.items():
            if param_value is not None and param_name in supported_params:
                provider_kwargs[param_name] = param_value

        # Добавляем остальные kwargs если они поддерживаются
        for key, value in kwargs.items():
            if key in supported_params:
                provider_kwargs[key] = value

        # Создаем structured output клиент
        new_instance.client = provider_method(**provider_kwargs)
        new_instance.__is_structured_output = True

        return new_instance

    async def astream(
        self,
        chat_history: list[dict[str, str] | BaseMessage] | None = None,
        system_prompt: str | BaseMessage | None = None,
        message: str | BaseMessage | None = None,
        moderation: bool = False,
        **kwargs,
    ) -> StreamResult:
        chat_for_model = PrepareChat(chat_history, system_prompt, message)

        await self.__moderation_check(moderation, chat_for_model)

        billing = StreamBillingDecorator(self.counter)
        full_output_text = ''

        async def content_generator():
            nonlocal full_output_text

            stream = self.client.astream(input=chat_for_model, **kwargs)
            async for chunk in stream:
                full_output_text += chunk.content
                yield chunk.content

            # Подсчитываем токены после завершения стрима
            await billing.count_input_tokens(chat_for_model)
            await billing.count_output_tokens(full_output_text)

            # Сохраняем историю чата
            ai_message = AIMessage(content=full_output_text)
            self._save_chat_history(chat_for_model, ai_message)

        return StreamResult(content_generator(), lambda: full_output_text)



from langchain_anthropic import ChatAnthropic

from llm.direction import TokenDirection
from llm.providers._base import BaseProvider, ModelConfig
from llm.test_connections import TestConnections
from llm.token_counters import TokenCounterFactory


class AnthropicProvider(BaseProvider):
    """Провайдер для моделей Anthropic"""

    @property
    def name(self) -> str:
        return 'Anthropic'

    def get_models(self) -> dict[str, ModelConfig]:
        """Возвращает конфигурации всех моделей Anthropic"""
        return {
            'claude-3-5-haiku-latest': ModelConfig(
                client_class=ChatAnthropic,
                token_counter=TokenCounterFactory().create_anthropic_counter(),
                test_connection=TestConnections().anthropic,
                pricing={
                    TokenDirection.ENCODE: 0.80 / 1_000_000,
                    TokenDirection.DECODE: 4.00 / 1_000_000,
                },
            ),
            'claude-3-7-sonnet-latest': ModelConfig(
                client_class=ChatAnthropic,
                token_counter=TokenCounterFactory().create_anthropic_counter(),
                test_connection=TestConnections().anthropic,
                pricing={
                    TokenDirection.ENCODE: 3.00 / 1_000_000,
                    TokenDirection.DECODE: 15.00 / 1_000_000,
                },
            ),
            'claude-opus-4-0': ModelConfig(
                client_class=ChatAnthropic,
                token_counter=TokenCounterFactory().create_anthropic_counter(),
                test_connection=TestConnections().anthropic,
                pricing={
                    TokenDirection.ENCODE: 15.00 / 1_000_000,
                    TokenDirection.DECODE: 75.00 / 1_000_000,
                },
            ),
            'claude-sonnet-4-0': ModelConfig(
                client_class=ChatAnthropic,
                token_counter=TokenCounterFactory().create_anthropic_counter(),
                test_connection=TestConnections().anthropic,
                pricing={
                    TokenDirection.ENCODE: 3.00 / 1_000_000,
                    TokenDirection.DECODE: 15.00 / 1_000_000,
                },
            ),
        }



from langchain.schema import AIMessage, BaseMessage, HumanMessage, SystemMessage


class PrepareChat:
    """Класс для подготовки чата в формате Langchain для модели.

    1. Данный класс всегда отдает список сообщений в формате Langchain.
    2. Если был отправлен только системный промпт, то после него добавляется пустое
        сообщение пользователя. Иначе некоторые модели не хотят работать с единственным
        системным промптом.

    Словарь chat_history должен иметь следующую структуру:
    ```
    [
        {'role': 'system' | 'user' | 'assistant', 'content': '...'},
        {'role': 'system' | 'user' | 'assistant', 'content': '...'},
    ]
    ```
    или
    ```
    [
        BaseMessage(content='...'),
        BaseMessage(content='...'),
    ]
    ```
    где BaseMessage - может быть любым классом из SystemMessage, HumanMessage, AIMessage


    Args:
        chat_history list[dict[str, str] | BaseMessage] | None: Список сообщений чата.
            По умолчанию None.
        system_prompt str | BaseMessage | None: Системный промпт. По умолчанию None.
        message str | BaseMessage | None: Сообщение. По умолчанию None.

    Returns:
        list[BaseMessage]: Список сообщений для модели
    """

    def __new__(
        cls,
        chat_history: list[dict[str, str] | BaseMessage] | None = None,
        system_prompt: str | BaseMessage | None = None,
        message: str | BaseMessage | None = None,
    ):
        instance = super().__new__(cls)
        return instance.prepare_chat(chat_history, system_prompt, message)

    def prepare_chat(
        self,
        chat_history: list[dict[str, str] | BaseMessage] | None = None,
        system_prompt: str | BaseMessage | None = None,
        message: str | BaseMessage | None = None,
    ) -> list[BaseMessage]:
        messages = []
        messages.extend(self._prepare_system_prompt(system_prompt))
        messages.extend(self._prepare_chat_history(chat_history))
        messages.extend(self._prepare_message(message))

        messages = self._add_empty_user_message_if_needed(messages)

        return messages

    def _add_empty_user_message_if_needed(
        self, messages: list[BaseMessage]
    ) -> list[BaseMessage]:
        """Добавляет пустое сообщение пользователя, если в списке только один системный
        промпт.

        Args:
            messages (list[BaseMessage]): Список сообщений

        Returns:
            list[BaseMessage]: Обновленный список сообщений
        """
        if len(messages) == 1 and isinstance(messages[0], SystemMessage):
            messages.append(HumanMessage(content=''))
        return messages

    def _prepare_system_prompt(
        self, system_prompt: str | BaseMessage | None
    ) -> list[BaseMessage]:
        messages = []
        if system_prompt:
            if isinstance(system_prompt, BaseMessage):
                messages.append(system_prompt)
            elif isinstance(system_prompt, str):
                messages.append(SystemMessage(content=system_prompt))
            else:
                raise ValueError(
                    'system_prompt должен быть либо строкой, либо объектом BaseMessage'
                )
        return messages

    def _prepare_chat_history(
        self, chat_history: list[dict[str, str] | BaseMessage] | None
    ) -> list[BaseMessage]:
        messages = []
        if chat_history:
            for msg in chat_history:
                if isinstance(msg, BaseMessage):
                    messages.append(msg)
                elif isinstance(msg, dict):
                    role = msg.get('role')
                    content = msg.get('content', '')
                    match role:
                        case 'system':
                            messages.append(SystemMessage(content=content))
                        case 'user':
                            messages.append(HumanMessage(content=content))
                        case 'assistant':
                            messages.append(AIMessage(content=content))
                        case _:
                            raise ValueError(f'Неизвестная роль сообщения: {role}')
                else:
                    raise ValueError(
                        'Элементы chat_history должны быть либо словарями, либо '
                        'объектами BaseMessage'
                    )
        return messages

    def _prepare_message(self, message: str | BaseMessage | None) -> list[BaseMessage]:
        messages = []
        if message:
            if isinstance(message, BaseMessage):
                messages.append(message)
            elif isinstance(message, str):
                messages.append(HumanMessage(content=message))
            else:
                raise ValueError(
                    'message должен быть либо строкой, либо объектом BaseMessage'
                )
        return messages



from langchain_xai import ChatXAI

from llm.direction import TokenDirection
from llm.providers._base import BaseProvider, ModelConfig
from llm.test_connections import TestConnections
from llm.token_counters import TokenCounterFactory


class XAIProvider(BaseProvider):
    """Провайдер для моделей XAI (Grok)"""

    @property
    def name(self) -> str:
        return 'XAI'

    def get_models(self) -> dict[str, ModelConfig]:
        """Возвращает конфигурации всех моделей XAI"""
        return {
            'grok-3-mini': ModelConfig(
                client_class=ChatXAI,
                token_counter=TokenCounterFactory().create_xai_counter(),
                test_connection=TestConnections().xai,
                pricing={
                    TokenDirection.ENCODE: 0.30 / 1_000_000,
                    TokenDirection.DECODE: 0.50 / 1_000_000,
                },
            ),
            'grok-3': ModelConfig(
                client_class=ChatXAI,
                token_counter=TokenCounterFactory().create_xai_counter(),
                test_connection=TestConnections().xai,
                pricing={
                    TokenDirection.ENCODE: 3.00 / 1_000_000,
                    TokenDirection.DECODE: 15.00 / 1_000_000,
                },
            ),
            'grok-3-fast': ModelConfig(
                client_class=ChatXAI,
                token_counter=TokenCounterFactory().create_xai_counter(),
                test_connection=TestConnections().xai,
                pricing={
                    TokenDirection.ENCODE: 5.00 / 1_000_000,
                    TokenDirection.DECODE: 25.00 / 1_000_000,
                },
            ),
        }



from ._base import BaseProvider  # noqa: F401
from ._factory import ProviderFactory  # noqa: F401

__all__ = ['BaseProvider', 'ProviderFactory']

try:
    from .openai import OpenAIProvider  # noqa: F401

    __all__.append('OpenAIProvider')
except ImportError:
    pass

try:
    from .gigachat import GigaChatProvider  # noqa: F401

    __all__.append('GigaChatProvider')
except ImportError:
    pass

try:
    from .anthropic import AnthropicProvider  # noqa: F401

    __all__.append('AnthropicProvider')
except ImportError:
    pass

try:
    from .google import GoogleProvider  # noqa: F401

    __all__.append('GoogleProvider')
except ImportError:
    pass

try:
    from .xai import XAIProvider  # noqa: F401

    __all__.append('XAIProvider')
except ImportError:
    pass

try:
    from .deepseek import DeepSeekProvider  # noqa: F401

    __all__.append('DeepSeekProvider')
except ImportError:
    pass

try:
    from .cerebras import CerebrasProvider  # noqa: F401

    __all__.append('CerebrasProvider')
except ImportError:
    pass

try:
    from .openrouter import OpenRouterProvider  # noqa: F401

    __all__.append('OpenRouterProvider')
except ImportError:
    pass
